
# Goal: Calculate rainfall amount cues for all combinations of n1 & n2 days.
# Requires values for n1 & n2 & input files cd & rain.
# cd holds census, date.mean & ndays & is ordered chronologically.
#    census values are increasing integers (missing integers are accomodated) 
#         (first row predates first census by 1 census interval),
#    date.mean (format is "YYYY-MM-DD HH:MM:SS"),
#    ndays is the number of days since 1980-01-01.
# rain holds V1 to V361 & is ordered chronologically.
#    VX is cumulative rainfall (mm) over X days before V361 (1<=X<=360),
#    V361 is number of days since 1980-01-01.
# The output file holds census, ndays, N1, N2, mean.rain & max.rain.
#    census values are increasing integers 
#    ndays is the mean date of the census as number of days since 1980-01-01.
#    N1 is number of days between flower fall & the end of the proximate cue
#    N2 is number of days over which the proximate cue accumulates
#    Mean.100 is average daily rain from N1+1 to N1+N2 days earlier (mm x 100).
#    Max.100 is the maximum value of Mean.100 (mm x 100).

setwd("C:/Users/wrightj/Dropbox/Clouds/RMSE/Dryad")
outfile = "Rain_Amount_observed.txt"

# Define n1 & n2. 
N1=seq(1,210,1)
N2=seq(1,125,1)

# Input census mean dates & select censuses to begin phenological years (BPY). 
# BPY is the census that Begins a Phenological Year preceding the current census.
cd = read.table("CensusMeanDates.txt",header=T,sep="\t")
cd$Di = NA # initialize first day of census
for (i in 2:nrow(cd)) {cd$Di[i] = cd$ndays[i-1]} # define first day of census
# Select first census of each year to begin a phenological year.
cd$time = strptime(cd$date.mean,format="%Y-%m-%d %H:%M:%S")
cd$mark = T
for (i in 2:dim(cd)[1]) {cd$mark[i] = ifelse(cd$time$year[i-1]==cd$time$year[i], F, T)}
BeginYr = cd$census[cd$mark==T]
cd = cd[,c("census","ndays","Di")] # remove variables that are not used again
cd$BPY = BeginYr[length(BeginYr)]
for (i in 1:nrow(cd)) {
  for (j in 2:length(BeginYr)) {
    cd$BPY[i] = ifelse(cd$census[i]>BeginYr[j-1] & cd$census[i]<=BeginYr[j], BeginYr[j-1], cd$BPY[i])
  }
}
cd = cd[cd$census>=6,] # flower records begin with census 7
cd = as.matrix(cd)

# Input the environmental file.
rain = read.table("Lookup_Rain.txt",header=T,sep="\t")
rain = rain[,c(1:(max(N1)+max(N2)),361)] # remove unused columns
rain = rain[rain[,ncol(rain)]>=cd[1,2] & rain[,ncol(rain)]<=cd[nrow(cd),2],] # remove unused rows
rain = as.matrix(rain)

# Create look up table for all pairwise values of N1 & N2.
result = matrix(nrow=(max(N1)-min(N1)+1)*(max(N2)-min(N2)+1)*(nrow(cd)-1),ncol=7) # Note nrow(cd)-1 
counter = 0
ndays = rain[,"V362"]
Di = cd[,"Di"]
Df = cd[,"ndays"]
for (n2 in min(N2):max(N2)) {
  print(n2)
  for (n1 in min(N1):max(N1)) {
    for (C in 2:nrow(cd)) {
      counter = counter+1
      condition = ndays>=Di[C] & ndays<Df[C]
      v = (rain[condition,(n1+n2)] - rain[condition,n1])/n2
      avg = mean(v)
      v = max(v)
      result[counter,] = c(cd[C,"census"],cd[C,"ndays"],cd[C,"BPY"],n1,n2,avg,v)      
    }
  }
}
result=as.data.frame(result)
colnames(result) = c("census","ndays","BPY","n1","n2","mean.rain","max.rain")
result$census = as.integer(result$census)
result$ndays = as.integer(result$ndays)
result$BPY = as.integer(result$BPY)
result$n1 = as.integer(result$n1)
result$n2 = as.integer(result$n2)
result$Mean.100 = as.integer(100*result$mean.rain)
result$Max.100 = as.integer(100*result$max.rain)
write.table(result,outfile,row.names=F,sep="\t")
