
# Goal: Calculate rainfall slope cues for all combinations of n1 & n2 days.
# Requires values for n1 & n2 & input files cd & daily rainfall.
# cd holds census, date.mean & ndays & is ordered chronologically.
#    census values are increasing integers (missing integers are accomodated) 
#         (first row predates first census by 1 census interval),
#    date.mean (format is "YYYY-MM-DD HH:MM:SS"),
#    ndays is the number of days since 1980-01-01.
# Daily rainfall holds ndays & rain (mm) & is ordered chronologically.
#     ndays is the mean date of the census as number of days since 1980-01-01.
#     daily rainfall (mm).
# The output file holds census, ndays, N1, N2, Mean.100 & Max.100.
#     census values are increasing integers 
#     ndays is the mean date of the census as number of days since 1980-01-01.
#     N1 is the number of days between flower fall & the end of the proximate cue
#     N2 is the number of days over which the proximate cue accumulates
#     Mean.100 is the average over the days in the census interval of the slope
#       of the rain-ndays relationship from N1+1 to N1+N2 days earlier (mm/day). 
#     Max.100 is the maximum over the days in the census interval of the slope 
#       of the rain-ndays relationship from N1+1 to N1+N2 days earlier (mm/day).

setwd("C:/Users/wrightj/Dropbox/Clouds/RMSE/Dryad")
infile1 = "CensusMeanDates.txt"
infile2 = "Daily_Rainfall.txt" # ndays, MJm2 & time
outfile = "Rain_Slope_Observed.txt"

# Define n1 & n2. 
N1=seq(1,210,1)
N2=seq(1,60,1)

# Input census mean dates (cd) and add begin phenological years (BPY). 
# BPY holds the census that Begins a Phenological Year preceding the current census.
cd = read.table(infile1,header=T,sep="\t") # Input census, date.mean & ndays
cd$Di = NA # initialize first day of census
for (i in 2:nrow(cd)) {cd$Di[i] = cd$ndays[i-1]} # define first day of census
# Select the first census of the year to begin phenological years.
cd$time = strptime(cd$date.mean,format="%Y-%m-%d %H:%M:%S")
cd$mark = T
for (i in 2:dim(cd)[1]) {cd$mark[i] = ifelse(cd$time$year[i-1]==cd$time$year[i], F, T)}
BeginYr = cd$census[cd$mark==T]
cd = cd[,c("census","ndays","Di")] # remove variables that are not used again
cd$BPY = BeginYr[length(BeginYr)]
for (i in 1:nrow(cd)) {
  for (j in 2:length(BeginYr)) {
    cd$BPY[i] = ifelse(cd$census[i]>BeginYr[j-1] & cd$census[i]<=BeginYr[j], BeginYr[j-1], cd$BPY[i])
  }
}
cd = cd[cd$census>=6,] # flower records begin with census 7
cd = as.matrix(cd)

# Create intermediate look up table of slopes for all combinations of ndays & N2.
Rain = read.table(infile2,header=T,sep="\t") # Input ndays, rain & time (time is not used) 
ndays = Rain[,"ndays"]
rain = Rain[,"rain"]
r = matrix(nrow=(max(N2)-min(N2)+1)*(dim(Rain)[1]-max(N2)),ncol=3)  
counter = 0
for (i in ((ndays[1]+max(N2)):ndays[length(ndays)])) {
  for (j in N2) {
    counter = counter+1
    condition = ndays %in% seq(i-j,i,1)
    m = lm(rain[condition] ~ ndays[condition])
    r[counter,] = c(i,j,m$coefficients[2])
  }
}
colnames(r) = c("ndays","n2","slope")

# Create look up table for all pairwise values of N1 & N2.
result = matrix(nrow=length(N1)*length(N2)*(nrow(cd)-1),ncol=7) # Note nrow(cd)-1 
counter = 0
ndays = unique(r[,"ndays"])
Di = cd[,"Di"]
Df = cd[,"ndays"]
for (n2 in N2) {
  print(n2)
  r.n2 = r[r[,"n2"]==n2,]
  for (n1 in N1) {
    for (C in 2:nrow(cd)) {
      counter = counter+1
      condition = ndays>=(Di[C]-n1) & ndays<(Df[C]-n1)
      v = r.n2[condition,"slope"]
      avg = mean(v)
      v = max(v) 
      result[counter,] = c(cd[C,"census"],cd[C,"ndays"],cd[C,"BPY"],n1,n2,avg,v)      
    }
  }
}
result=as.data.frame(result)
colnames(result) = c("census","ndays","BPY","n1","n2","Mean.100","Max.100")
result$census = as.integer(result$census)
result$ndays = as.integer(result$ndays)
result$BPY = as.integer(result$BPY)
result$n1 = as.integer(result$n1)
result$n2 = as.integer(result$n2)
write.table(result,outfile,row.names=F,sep="\t")
