
# Goal: Calculate solar slope cues for all combinations of n1 & n2 days.
# Requires values for n1 & n2 & input files cd & daily solar irradiance.
# cd holds census, date.mean & ndays & is ordered chronologically.
#   census values are increasing integers (missing integers are accomodated) 
#        (first row predates first census by 1 census interval),
#   date.mean is not used, &
#   ndays is the number of days since 1980-01-01.
# Daily solar irradiance holds ndays & MJ/m2 & is ordered chronologically.
#   ndays is the mean date of the census as number of days since 1980-01-01.
#   MJm2 is MJ per m2 per day.
# The output file holds census, ndays, N1, N2, Mean.100 & Max.100.
#   census values are increasing integers 
#   ndays is the mean date of the census as number of days since 1980-01-01.
#   N1 is the number of days between flower fall & the end of the proximate cue
#   N2 is the number of days over which the proximate cue accumulates
#   Mean.100 is the average over the days in the census interval of the slope of 
#     the MJm2-ndays relationship from N1+1 to N1+N2 days earlier (MJ/m2/day). 
#   Max.100 is the maximum over the days in the census interval of the slope of 
#     the MJm2-ndays relationship from N1+1 to N1+N2 days earlier (MJ/m2/day).

setwd("C:/Users/wrightj/Dropbox/Clouds/RMSE/Dryad")
infile1 = "CensusMeanDates.txt"
infile2 = "Daily_Solar_Radiation.txt" # ndays, MJm2 & time
outfile = "Sun_Slope_Observed.txt"

# Define n1 & n2. 
N1=seq(1,210,1)
N2=seq(1,60,1)

# Input census mean dates (cd) and add BPY. 
# BPY holds the final census that Begins a Phenological Year preceding the current census.
# Phenological years begin in late Oct, when solar inputs are at their annual low.
# BeginYr holds the final censuses from each October.
cd = read.table(infile1,header=T,sep="\t") # Input census, date.mean & ndays (date.mean is not used)
cd = cd[,c(1,3)] # remove unused date variable
cd$Di = NA # initialize first day of census 
for (i in 2:nrow(cd)) {cd$Di[i] = cd$ndays[i-1]} # define first day of census
BeginYr = c(0,43,96,147,199,251,303,355,407,460,512,564,616,668,721,773,825,
            877,929,981,1034,1086,1138,1190,1242,1295,1347,1399,1451,1503,1555)
cd$BPY = BeginYr[length(BeginYr)]
for (i in 1:nrow(cd)) {
  for (j in 2:length(BeginYr)) {
    cd$BPY[i] = ifelse(cd$census[i]>BeginYr[j-1] & cd$census[i]<=BeginYr[j], BeginYr[j-1], cd$BPY[i])
  }
}
cd = cd[cd$census>=6,] # flower records begin with census 7
cd = as.matrix(cd)

# Create intermediate look up table of slopes for all combinations of ndays & N2.
sun = read.table(infile2,header=T,sep="\t") # Input ndays, MJm2 & time (time is not used) 
ndays = sun[,"ndays"]
MJm2 = sun[,"MJm2"]
r = matrix(nrow=(max(N2)-min(N2)+1)*(dim(sun)[1]-max(N2)),ncol=3)  
counter = 0
for (i in ((ndays[1]+max(N2)):ndays[length(ndays)])) {
  for (j in N2) {
    counter = counter+1
    condition = ndays %in% seq(i-j,i,1)
    m = lm(MJm2[condition] ~ ndays[condition])
    r[counter,] = c(i,j,m$coefficients[2])
  }
}
colnames(r) = c("ndays","n2","slope")

# Create look up table for all pairwise values of N1 & N2.
result = matrix(nrow=length(N1)*length(N2)*(nrow(cd)-1),ncol=7) # Note nrow(cd)-1 
counter = 0
ndays = unique(r[,"ndays"])
Di = cd[,"Di"]
Df = cd[,"ndays"]
for (n2 in N2) {
  print(n2)
  r.n2 = r[r[,"n2"]==n2,]
  for (n1 in N1) {
    for (C in 2:nrow(cd)) {
      counter = counter+1
      condition = ndays>=(Di[C]-n1) & ndays<(Df[C]-n1)
      v = r.n2[condition,"slope"]
      avg = mean(v)
      v = max(v) 
      result[counter,] = c(cd[C,"census"],cd[C,"ndays"],cd[C,"BPY"],n1,n2,avg,v)      
    }
  }
}
result=as.data.frame(result)
colnames(result) = c("census","ndays","BPY","n1","n2","Mean.100","Max.100")
result$census = as.integer(result$census)
result$ndays = as.integer(result$ndays)
result$BPY = as.integer(result$BPY)
result$n1 = as.integer(result$n1)
result$n2 = as.integer(result$n2)
write.table(result,outfile,row.names=F,sep="\t")
